********************************************************************************
*
*Compute firm and relationship level exposure measures 
*
*Output: Firm level and network level data frames which can be used in Python
*********************************************************************************

	frame change default
	
	cap frame drop firms
	cap frame drop network
	
********************************************************************************
*Load firm level data
********************************************************************************
	frame create firms
	frame change firms
	
	use "$comppanel/Comprehensive_firm_panel_monthly.dta", clear

********************************************************************************
*Compute supply side firm-level shares
********************************************************************************
		

	***************
	*Specify variables that are needed 
	global flows total_exports total_imports sales_network_monthly sales total_sales total_salary purchases_network_monthly purchases  total_purchases imports exports
	keep tax_payer_id year date isic_class_code isic_group_code sector_id $flows	
	
	xtset tax_payer_id date
		rename date networkdate
		rename year networkyear	
	***************
	*Compute 12-months rolling values
	foreach var of varlist $flows{
		tsegen `var'12=rowtotal(L(1/12).`var')
		replace `var'=`var'12
		drop `var'12
	}
	***************
	*Firm-level direct exposure measure (demand)
	
		*Exposure to exports
		gen r_iX = total_exports/sales
			replace r_iX = 0 if r_iX==. // cases where sales are zero
			label variable r_iX "Revenue share of exports"

		*Exposure to exports based on customs
		gen revenues_customs = total_sales + exports
			label variable revenues_customs "Total revenue (domestic sales + exports from customs)"
		gen r_iXc = exports/revenues_customs
			replace r_iXc = 0 if r_iXc==. // cases where sales are zero
			label variable r_iXc "Revenue share of exports (customs)"

		*Exposure to domestic household expenditure
		gen sales_H = total_sales - sales_network
				replace sales_H = 0 if sales_H<0
				label variable sales_H "Domestic sales outside firm network"
				
		gen r_iH = sales_H/sales
			replace r_iH = 0 if r_iH==. // cases where sales are zero
			sum r_iH
			label variable r_iH "Domestic sales outside network revenue share"

		gen r_iHc = sales_H/sales
			replace r_iHc = 0 if r_iH==. // cases where sales are zero
			sum r_iHc
			label variable r_iHc "Domestic sales outside network revenue share (+ customs)"

	****************
	*Prepare for merging with network data
	
	preserve
		keep tax_payer_id networkdate networkyear sales sales_network_monthly revenues_customs r_iX r_iH r_iHc r_iXc
		rename tax_payer_id supplier_id

		compress 
		
		tempfile suppliervars
		save `suppliervars', replace
	restore
	
********************************************************************************
*Compute demand side firm-level shares
********************************************************************************

	***************
	*Firm-level direct exposure measure
				
		*Exposure to imports
			sum total_imports purchases_network total_salary total_purchases
		gen total_inputs = total_salary + total_imports + total_purchases
			label variable total_inputs "Inputs (labour cost + domestic purchases + imports)"

		gen total_inputs_customs = total_salary + imports + total_purchases
			label variable total_inputs "Inputs (labour cost + domestic purchases + imports customs)"

		gen r_iM = total_imports/total_inputs 
			replace r_iM = 0 if r_iM==. 
			label variable r_iM "Input share of imports"

		gen r_iMc = imports/total_inputs_customs 
			replace r_iMc = 0 if r_iMc==. 
			label variable r_iMc "Input share of imports (customs)"

		compress
		
	****************
	*Prepare for merging with network data
	
	preserve 
		keep tax_payer_id networkdate networkyear total_inputs purchases_network_monthly total_inputs_customs r_iM r_iMc
		rename tax_payer_id buyer_id

		
		compress
		
		tempfile buyervars
		save `buyervars', replace
	restore
	
********************************************************************************
*Load network data
********************************************************************************
	
	frame create network
	frame change network
	
	use "$comppanel/Rolling_network_panel_monthly.dta", clear
	cap drop _merge


	*************
	*Add revenue and input shares from firm-level data
	
	merge m:1 supplier_id networkdate using `suppliervars'
		drop if _merge==2 // firms that are not suppliers
		cap drop _merge 
	
	merge m:1 buyer_id networkdate using `buyervars'
		drop if _merge==2 // firms that are not buyers
		cap drop _merge
	

********************************************************************************
*Compute firm-to-firm exposure measures
********************************************************************************

*Downstream

	***************
	*Firm-to-firm exposure measure downstream r_ij

	****
	*Based on VAT data 
	
	gen r_ij = transaction_volume/sales
		label variable r_ij "Revenue share of j's sales to i"
		
		count if r_ij>1

	egen sharesum=total(r_ij), by(supplier_id networkdate) // some cases with shares greater than 1

	replace sharesum=sharesum+r_iX+r_iH
		count if sharesum>1.0001
	replace r_ij=r_ij/sharesum if sharesum>1 & sharesum!=.
	replace r_iX=r_iX/sharesum if sharesum>1 & sharesum!=.
	replace r_iH=r_iH/sharesum if sharesum>1 & sharesum!=.
		cap drop sharesum

	***
	*Based on customs data
	gen r_ijc = transaction_volume/revenues_customs
		label variable r_ijc "Revenue share of j's sales to i (+ customs)"

	egen sharesum=total(r_ijc), by(supplier_id networkdate) // some cases with shares greater than 1

	replace sharesum=sharesum+r_iXc+r_iHc
		count if sharesum>1.0001
	replace r_ijc=r_ijc/sharesum if sharesum>1 & sharesum!=.
	replace r_iXc=r_iXc/sharesum if sharesum>1 & sharesum!=.
	replace r_iHc=r_iHc/sharesum if sharesum>1 & sharesum!=.

		cap drop sharesum
		
		
*Upstream 

	***************
	*Firm-to-firm exposure measure upstream s_ij
	
		*Fill blanks for hand full of firms where no input info is available
		egen inputs_proxy=total(transaction_volume), by(buyer_id networkdate)
		replace total_inputs = inputs_proxy if total_inputs==.
		replace total_inputs_customs = inputs_proxy if total_inputs_customs==.
		cap drop inputs_proxy
	
	***
	*Based on VAT data
	gen s_ij = transaction_volume/total_inputs
		label variable s_ij "Input share of j's purchases from i"
	
		*sum s_ij, detail
		count if s_ij>1 // no adjustments needed
	
	egen sharesum=total(s_ij), by(buyer_id networkdate) // some cases with shares greater than 1

	replace r_iM=0 if r_iM==.
	replace sharesum=sharesum+r_iM
		sum sharesum, detail
		count if sharesum>1 & sharesum!=.
	replace s_ij=s_ij/sharesum if sharesum>1 & sharesum!=.
	replace r_iM=r_iM/sharesum if sharesum>1 & sharesum!=.
		cap drop sharesum

	***
	*Based on customs data
	gen s_ijc = transaction_volume/total_inputs_customs
		label variable s_ijc "Input share of j's purchases from i"

	egen sharesum=total(s_ijc), by(buyer_id networkdate) // some cases with shares greater than 1

	replace r_iMc=0 if r_iMc==.
	replace sharesum=sharesum+r_iMc
		sum sharesum, detail
		count if sharesum>1 & sharesum!=.
	replace s_ijc=s_ijc/sharesum if sharesum>1 & sharesum!=.
	replace r_iMc=r_iMc/sharesum if sharesum>1 & sharesum!=.
		cap drop sharesum

********************************************************************************
*Select relevant variables
********************************************************************************

	keep supplier_id buyer_id networkdate r_ij* s_ij* r_iX* r_iH* r_iM*
	compress
	
********************************************************************************
*Prepare to merge back into firm data to correct exposure measures
********************************************************************************
		
		*Mainly necessary for suppliers, for buyers only correcting low decimals
		preserve
			keep supplier_id networkdate r_iX* r_iH*
			bys supplier_id networkdate: keep if _n==1
			rename supplier_id tax_payer_id
			
			tempfile suppliervarsC
			save `suppliervarsC', replace
		restore	

		preserve
			keep buyer_id networkdate r_iM*
			bys buyer_id networkdate: keep if _n==1
			rename buyer_id tax_payer_id
			
			tempfile buyervarsC
			save `buyervarsC', replace
		restore	

********************************************************************************
*Correct firm level data
********************************************************************************
	
	frame change firms
	
	*Update r_iX and r_iH values using corrected shares based on network data
	merge 1:1 tax_payer_id networkdate using `suppliervarsC', replace update
		drop if _merge==2
		cap drop _merge

	merge 1:1 tax_payer_id networkdate using `buyervarsC', replace update
		*drop if _merge==2
		cap drop _merge
